<?php
/**
 * $Horde: nic/lib/Nic.php,v 1.37 2006/11/14 18:14:30 chuck Exp $
 *
 * @package Nic
 */

/**
 * A measure of how much of somethings is being used e.g. percentage
 * of CPU usage.
 */
define('NIC_DATA_GAUGE', 701);

/**
 * A Rate is the number of times something happens per second
 * e.g. queries per second.
 */
define('NIC_DATA_RATE', 702);

/**
 * A Counter increases as things happen e.g. amount of data
 * transferred.
 */
define('NIC_DATA_COUNTER', 703);

/**
 * An integer data type is the number things that exist e.g. number of
 * active connections or sessions.
 */
define('NIC_DATA_INTEGER', 704);

/**
 * Device is responding normally.
 */
define('NIC_STATUS_NORMAL', 810);

/**
 * Device is in a warning state.
 */
define('NIC_STATUS_WARN', 820);

/**
 * Device is in a fault state.
 */
define('NIC_STATUS_FAULT', 830);

/**
 * Device is in an unknown state.
 */
define('NIC_STATUS_UNKNOWN', 840);

/**
 * Common functions for Nic.
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @since   Nic 0.2
 * @package Nic
 */
class Nic {

    /**
     * Fetches all storable details from devices and add them to the storage
     * backend.
     */
    function fetchAndStore()
    {
        global $conf, $devices, $rrdtools;

        require_once NIC_BASE . '/lib/Device.php';
        require_once NIC_BASE . '/lib/Storage.php';
        require_once NIC_BASE . '/lib/Status.php';

        $status_storage = &Nic_Status::singleton();
        $data_storage   = &Nic_Storage::singleton();
        $rrd_storage    = &Nic_Storage::singleton("rrd");

        foreach ($devices as $deviceid => $device) {
            printf(_("Fetching status for %s on %s"),
                   $deviceid,
                   $device['params']['hostname']);
            $dev = &Nic_Device::singleton($device['driver'], $device['params']);

            $now = time();
            $status = $dev->getStatus();
            $status_storage->setStatus($now,
                                       $device['params']['hostname'],
                                       $device['params']['subsystem'],
                                       $status['status'],
                                       $status['message']);
            echo " done\n";

            if ($status['status'] != NIC_STATUS_NORMAL) {
                printf(_("Skip fetching data for %s on %s"),
                       $deviceid,
                       $device['params']['hostname']);
                echo "\n";
            } else {
                $rrd_files = array();

                printf(_("Fetching data for %s on %s"),
                       $deviceid,
                       $device['params']['hostname']);
                $data = $dev->retrieveStoreableData();
                foreach ($data as $data_item => $data_value) {
                    $data_storage->add($now,
                                       $device['params']['hostname'],
                                       $device['params']['subsystem'],
                                       $data_item,
                                       $data_value['value']);

                    if (isset($device['params']['oids'])) {
                        // Cycle through OIDs to find the filename to use for
                        // this data.
                        foreach ($device['params']['oids'] as $oid) {
                            if (array_key_exists('rrdtool', $oid) &&
                                in_array($data_item, $oid)) {
                                $rrdtool = $rrdtools[$oid['rrdtool']];
                                $rrd_files[$rrdtool['file']][] = $data_value['value'];
                            }
                        }
                    }
                }

                // Update the RRD Files.
                foreach ($rrd_files as $rrd_file => $rrd_data) {
                    $result = $rrd_storage->add($rrd_file, $rrd_data);

                    // Somehow log/display that there was a problem with the
                    // RRD.
                    // TODO: Make this a little better, log it, etc.
                    if (is_a($result, 'PEAR_Error')) {
                        echo " \n" . $result->getMessage();
                    }
                }
                echo " done\n";
            }
        }
    }

    /**
     * Purges old data and status records.
     */
    function purge()
    {
        global $conf;

        $keepseconds = $conf['purge']['keepdays'] * 24 * 60 * 60;
        $removeolder = time() - $keepseconds;

        // Set threshold for removing records.
        $threshold = $removeolder - 86400;

        require_once NIC_BASE . '/lib/Storage.php';
        require_once NIC_BASE . '/lib/Status.php';

        $data_storage   = &Nic_Storage::singleton();

        // Only remove if threshold is exceeded.
        if ($data_storage->oldest() < $threshold) {
            echo 'Purging records from before '. $removeolder;

            if ($result = $data_storage->purge($removeolder)) {
                $status_storage = &Nic_Status::singleton();
                $result = $status_storage->purgeStatus($removeolder);
            }

            if (!is_a($result, 'PEAR_error')) {
                echo " done\n";
            } else {
                echo " FAILED\n";
            }
        }
    }

    function getStatusHTML($status)
    {
        global $hostname, $subsystem, $registry;

        switch ($status['status']) {
        case NIC_STATUS_NORMAL:
            $color = '#009900';
            $status_text = _("Working Normally");
            break;

        case NIC_STATUS_WARN:
            $color = '#ff8000';
            $status_text = _("Warning");
            break;

        case NIC_STATUS_FAULT:
            $color = '#ff0000';
            $status_text = _("Fault");
            break;

        default:
            return '';
        }

        $img = Horde::url($registry->get('webroot', 'horde') . '/services/images/pixel.php');
        $img = Util::addParameter($img, 'c', $color);

        $url = Util::addParameter('subsystem.php', array('hostname' => $hostname,
                                                         'subsystem' => $subsystem));

        $comment = split("[\n]|[\r]|[\n\r]", rtrim($status['comment']));
        $comment = implode("\n", $comment);

        return Horde::linkTooltip(Horde::applicationUrl($url), $status_text, '', '', '', $comment) . '<img src="' . $img . '" class="status" alt="' . $status_text . '" /></a>';
    }

    /**
     * Builds Nic's list of menu items.
     */
    function getMenu($returnType = 'object')
    {
        global $conf;

        require_once 'Horde/Menu.php';

        $menu = new Menu();
        $menu->add(Horde::applicationUrl('status.php'), _("_Monitoring"), 'nic.png');

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}
